@@ -24,6 +24,10 @@
 
 	------------    Changes    ------------
 
+	2008-06-15  Still me
+
+	- Added audio mask
+	
 	2004-08-13  Still me
 
 	- changed oversampling to 4x
@@ -79,8 +83,8 @@
 #define EXPGAMMA  (2.4)
 
 int versionmajor=0;
-int versionminor=0;
-int versionpatch=4;
+int versionminor=1;
+int versionpatch=0;
 
 int NUMOVER=4;
 
@@ -136,14 +140,18 @@ int inverse=0;
 int oversample=0;
 int randphase=0;
 
+int masking=0;
+
 float lowfreq=0;
 float highfreq=100;
+float length=0;
 
 float vgamma=1.0;
 
 char *text=NULL;
 char *inputfilename=NULL;
 char *outputfilename=NULL;
+char *maskfilename=NULL;
 char *defaultoutputfilenamewav="output.wav";
 char *defaultoutputfilenameraw="output.raw";
 char *defaultoutputfilenameaiff="output.aiff";
@@ -155,7 +163,7 @@ void fft(float *fr, float *fi, int ldn, int is);
 
 void printblurb()
 	{
-	printf("Enscribe %d.%d.%d      (c) 2004 Jason Downer\n",versionmajor,versionminor,versionpatch);
+	printf("Enscribe %d.%d.%d      (c) 2004-2008 Jason Downer\n",versionmajor,versionminor,versionpatch);
 	printf("Homepage: http://jbd.zayda.net/enscribe/\n\n");
 	printf("This is ALPHA software.\n\n");
 	printf("Enscribe is free software, distributed under the General Public License.\n\n");
@@ -186,6 +194,7 @@ void printoptions()
 	printf("                           1 = 1024 (default)\n");
 	printf("                           2 = 2048\n");
 	printf("                           3 = 4096\n");
+	printf("                           4 = 8192\n");
 	printf("\n");
 //	printf("  -t text                Print text instead of image\n");
 //	printf("  -fs=[font size]        Sets text font size (0 to 4)\n");
@@ -219,6 +228,9 @@ void printoptions()
 	printf("  -double                64-bit floating point\n");
 	printf("  -ulaw                  U-Law\n");
 	printf("  -alaw                  A-Law\n");
+	printf("\n");
+	printf("  -mask filename         Use sound \"filename\" as FFT mask\n");
+	printf("  -length=[seconds]      Specify a length in seconds (may lose some detail!)\n");
 	}
 
 int readoptions(int argc, char** argv)
@@ -403,6 +415,44 @@ int readoptions(int argc, char** argv)
 						}
 					}
 				}//-hf
+			else if (strncmp(argv[i],"-length",7)==0)
+				{
+				if ( (strncmp(argv[i],"-length=",8)!=0) || (strlen(argv[i])<9) )
+					{
+					fprintf(stderr,"Error: You need to specify a length in seconds such as:\n");
+					fprintf(stderr,"-length=25.0\n");
+					exit(BAD_ARGS);
+					}
+				else 
+					{
+					strncpy(arg,&(argv[i][8]),255);
+					f=atof(arg);
+					if ((f<1) || (f>1000))
+						{
+						fprintf(stderr,"Error: You need to specify a length in seconds between 1 and 1000.\n");
+						exit(BAD_ARGS);
+						}
+					else
+						{
+						length=f;
+						}
+					}
+				}//-length
+			else if (strncmp(argv[i],"-mask",5)==0)
+				{
+				if ( i+1 >= argc )
+					{
+					fprintf(stderr,"Error: You need to specify a audio filename such as:\n");
+					fprintf(stderr,"-mask file.wav\n");
+					exit(BAD_ARGS);
+					}
+				else 
+					{
+					maskfilename=argv[i+1];
+					masking=1;
+					i++;
+					}
+				}//-mask
 			else if (strncmp(argv[i],"-rate",5)==0)
 				{
 				if ( (strncmp(argv[i],"-rate=",6)!=0) || (strlen(argv[i])<7) )
@@ -461,9 +511,9 @@ int readoptions(int argc, char** argv)
 					{
 					strncpy(arg,&(argv[i][4]),255);
 					j=atol(arg);
-					if ((j<0) || (j>4))
+					if ((j<0) || (j>7))
 						{
-						fprintf(stderr,"Error: You need to specify a transform size between 0 and 4.\n");
+						fprintf(stderr,"Error: You need to specify a transform size between 0 and 7.\n");
 						exit(BAD_ARGS);
 						}
 					else
@@ -629,7 +679,9 @@ int main(int argc, char** argv)
 	FILE *in;
 
 	SF_INFO out_info;
+	SF_INFO mask_info;
 	SNDFILE* out;
+	SNDFILE* maskfile;
 
 	int size, halfsize, quartersize, dblsize;
 	int startx,stopx;
@@ -641,7 +693,7 @@ int main(int argc, char** argv)
 	float inv_halfsize;
 	float inv_quartersize;
 
-	int c,x,x2,y,t,i;
+	int c,x,x2,y,t,i,j;
 	
 	float *realL;  
 	float *imagL;
@@ -658,6 +710,7 @@ int main(int argc, char** argv)
 	
 	float *lerp;
 	float *window;
+	float *mask;
 	
 	float ph0,ph1;
 	
@@ -735,6 +788,12 @@ int main(int argc, char** argv)
 		}
 	checkmem(temp,"temp");
 	
+	if (masking)
+		{	
+		mask=(float*)calloc(halfsize,sizeof(float));
+		checkmem(mask,"mask");
+		}
+	
   if (oversample)
 		{
 		bufL1=(float*)calloc(quartersize,sizeof(float));
@@ -775,6 +834,92 @@ int main(int argc, char** argv)
 			}
     }
 
+
+	if (masking)
+		{
+		int mc,ms,mcount=0,mcc=0;
+		float maxp=0;
+		float *mbuf;
+		fprintf(stderr,"Opening mask file\n");
+		maskfile=sf_open(maskfilename,SFM_READ,&mask_info);
+		if (maskfile==NULL)
+			{
+			fprintf(stderr,"Error opening mask file: %s\n",maskfilename);
+			perror(NULL);
+			exit(AUDIO_FILE_ERROR);
+			}
+
+		mc=mask_info.channels;
+		ms=size*mc;
+		mbuf=(float*)calloc(ms,sizeof(float));
+		checkmem(mbuf,"mbuf");
+
+		memset((void *)realL, 0, sizeof(float)*size);
+		memset((void *)imagL, 0, sizeof(float)*size);
+
+		do
+			{
+			mcount=sf_read_float(maskfile, mbuf, ms);
+			if (mcount!=ms) break;
+			mcc++;
+			if (mc==1)
+				{
+				for (i=0;i<size;i++)
+					realL[i]+=mbuf[i];
+				}
+			else if (mc==2)
+				{
+				for (i=0,j=0;i<size;i++,j+=2)
+					realL[i]+=mbuf[j]+mbuf[j+1];
+				}
+			else
+				{
+				fprintf(stderr,"Cannot read %d channel sound %s\n",mc,maskfilename);
+				exit(AUDIO_FORMAT_ERROR);
+				}
+			
+			}
+		while (mcount==ms); //redundant
+		fprintf(stderr,"Read %d chunks from %d channel sound %s\n",mcc,mc,maskfilename);
+		
+		if (mcc==0)
+			{		
+			fprintf(stderr,"Could not get enough audio data from %s!\nSwitching to non-masking mode!\n",maskfilename);
+			masking=0;
+			}
+		else
+			{
+			//Windowing
+			for (i=0;i<size;i++)
+				{
+				realL[i]*=sin((M_PI*(i))/size);
+				}
+
+			fprintf(stderr,"Performing FFT...\n");
+			fft(realL,imagL,ldn,+1);
+		
+			startx=(lowfreq/100.0)*halfsize; //redundant
+			stopx=(highfreq/100.0)*halfsize;
+
+			for (i=startx;i<stopx;i++)
+				{
+				mask[i]=sqrt(realL[i]*realL[i]+imagL[i]*imagL[i]);
+				if (mask[i]>maxp) maxp=mask[i];
+				}
+
+			for (i=0;i<startx;i++) mask[i]=0;
+			for (i=stopx;i<halfsize;i++) mask[i]=0;
+
+			if (maxp>0)
+				for (i=startx;i<stopx;i++)
+					{
+					mask[i]=(1<<transize)*40*mask[i]/maxp;
+					};
+			}
+		free(mbuf);
+		sf_close(maskfile);
+		}
+
 	if (!textmode)
 		{
 		char *lower;
@@ -881,12 +1026,20 @@ int main(int argc, char** argv)
 		h=stopx - startx;//???
 		}
 */
-	h2=h*scale;
+	if (length>0)
+		h2=(length*samplerate)/size;
+	else
+		{
+		h2=h*scale;
+		//h2=h*512/((float)w);
+		//h2=h >> transize;
+		}
 	if (oversample)
 		im2 = gdImageCreateTrueColor( halfsize, (int)(h2*4) );
 	else
 		im2 = gdImageCreateTrueColor( halfsize, (int)(h2) );
-	
+
+	fprintf(stderr,"Scaling Image\n");
 	gdImageCopyResampled(im2, im, startx, 0, 0, 0, stopx-startx, im2->sy, im->sx, im->sy);
 
 //-----------Setup Output-------------- 
@@ -926,6 +1079,8 @@ int main(int argc, char** argv)
 		exit(AUDIO_FORMAT_ERROR);
 		} 
 
+fprintf(stderr,"Opening output file\n");
+
 	if (audioformat==STDOUTAUDIO)
 		{
 		out=sf_open_fd(STDOUT_FILENO, SFM_WRITE, &out_info, 0);
@@ -968,6 +1123,8 @@ int main(int argc, char** argv)
 		buftblR[2]=bufR3;
 		buftblR[3]=bufR4;
 		}
+	fprintf(stderr,"Building output audio file\n");
+
 	//-----------Main loop------------//
 	for (y = 0; y < im2->sy; y++)
 		{
@@ -1077,6 +1234,19 @@ int main(int argc, char** argv)
 				imagR[x] =  cr * sin(ph0);
 				realR[x] =  cr * cos(ph0);
 				}
+			
+			if (masking)
+				{
+				imagL[x] *=  mask[x];
+				realL[x] *=  mask[x];
+				if (channels==2)
+					{
+					imagR[x] *=  mask[x];
+					realR[x] *=  mask[x];
+					}
+				}
+			
+			
 			}
 		
 		fft(realL,imagL,ldn,-1);
